@@ -2,7 +2,7 @@
 (*                                                                        *)
 (*  This file is part of Calendar.                                        *)
 (*                                                                        *)
-(*  Copyright (C) 2003-2010 Julien Signoles                               *)
+(*  Copyright (C) 2003-2011 Julien Signoles                               *)
 (*                                                                        *)
 (*  you can redistribute it and/or modify it under the terms of the GNU   *)
 (*  Lesser General Public License version 2.1 as published by the         *)
@@ -33,19 +33,19 @@ module type S = sig
   val from_string : string -> t
 end
 
-let day_name = 
+let day_name =
   ref (function
 	 | Date.Sun -> "Sunday"
 	 | Date.Mon -> "Monday"
 	 | Date.Tue -> "Tuesday"
 	 | Date.Wed -> "Wednesday"
-	 | Date.Thu -> "Thurday"
+	 | Date.Thu -> "Thursday"
 	 | Date.Fri -> "Friday"
 	 | Date.Sat -> "Saturday")
 
 let name_of_day d = !day_name d
 
-let short_name_of_day d = 
+let short_name_of_day d =
   let d = name_of_day d in
   try String.sub d 0 3 with Invalid_argument _ -> d
 
@@ -66,7 +66,7 @@ let month_name =
 
 let name_of_month m = !month_name m
 
-let short_name_of_month m = 
+let short_name_of_month m =
   let m = name_of_month m in
   try String.sub m 0 3 with Invalid_argument _ -> m
 
@@ -96,13 +96,13 @@ let print_number fmt pad k n =
 
 let bad_format s = raise (Invalid_argument ("bad format: " ^ s))
 
-let not_match f s = 
+let not_match f s =
   raise (Invalid_argument (s ^ " does not match the format " ^ f))
 
 let gen_month_of_name f fmt name =
-  let rec aux i = 
+  let rec aux i =
     if i = 0 then not_match fmt name
-    else if f (Date.month_of_int i) = name then i 
+    else if f (Date.month_of_int i) = name then i
     else aux (i-1)
   in
   aux 12
@@ -111,9 +111,9 @@ let month_of_name = gen_month_of_name name_of_month "%b"
 let month_of_short_name = gen_month_of_name short_name_of_month "%B"
 
 let gen_day_of_name f fmt name =
-  let rec aux i = 
+  let rec aux i =
     if i = 0 then not_match fmt name
-    else if f (Date.day_of_int i) = name then i 
+    else if f (Date.day_of_int i) = name then i
     else aux (i-1)
   in
   aux 7
@@ -121,14 +121,18 @@ let gen_day_of_name f fmt name =
 let day_of_name = gen_day_of_name name_of_day "%a"
 let day_of_short_name = gen_day_of_name short_name_of_day "%A"
 
-let word_regexp = ref (Str.regexp "[a-zA-Z]+")
+let word_regexp = ref (Re.Str.regexp "[a-zA-Z]+")
 
 let set_word_regexp r = word_regexp := r
 
+type 'a second_builder =
+  | Int of (int -> int -> int -> int -> int -> int -> 'a)
+  | Float of (int -> int -> int -> int -> int -> float -> 'a)
+
 (* [Make] creates a printer from a small set of functions. *)
 module Make(X : sig
 	      type t
-	      val make : int -> int -> int -> int -> int -> int -> t
+	      val make : t second_builder
 	      val from_business: Date.year -> int -> Date.day -> t
 	      val default_format : string
 	      val hour : t -> int
@@ -175,9 +179,9 @@ struct
     let seconds_since_1970 = lazy (X.seconds_since_1970 x) in
     let print_char c = Format.pp_print_char fmt c in
     let print_int pad k n = print_number fmt pad k (Lazy.force n) in
-    let print_string pad s = 
-      let pad s = match pad with 
-	| Uppercase -> String.uppercase s 
+    let print_string pad s =
+      let pad s = match pad with
+	| Uppercase -> String.uppercase_ascii s
 	| Empty | Zero | Blank -> s
       in
       Format.pp_print_string fmt (pad (Lazy.force s))
@@ -189,7 +193,7 @@ struct
       print_char ':';
       print_int pad 10 second in
     let rec parse_option i pad =
-      let parse_char c = 
+      let parse_char c =
 	let jump = ref 0 in
 	begin match c with
 	| '%' -> print_char '%'
@@ -209,7 +213,7 @@ struct
 	    print_int pad 1000 year
 	| 'C' -> print_int pad 10 century
 	| 'd' -> print_int pad 10 day_of_month
-	| 'D' -> 
+	| 'D' ->
 	    print_int pad 10 int_month;
 	    print_char '/';
 	    print_int pad 10 day_of_month;
@@ -231,9 +235,9 @@ struct
 	| 'M' -> print_int pad 10 minute
 	| 'n' -> print_char '\n'
 	| 'p' -> print_string pad apm
-	| 'P' -> 
-	    Format.pp_print_string fmt (String.lowercase (Lazy.force apm))
-	| 'r' -> 
+	| 'P' ->
+	    Format.pp_print_string fmt (String.lowercase_ascii (Lazy.force apm))
+	| 'r' ->
 	    print_time pad shour;
 	    print_char ' ';
 	    print_string pad apm
@@ -245,26 +249,26 @@ struct
 	| 'S' -> print_int pad 10 second
 	| 't' -> print_char '\t'
 	| 'T' -> print_time pad hour
-	| 'V' | 'W' -> print_int pad 10 week	     
+	| 'V' | 'W' -> print_int pad 10 week
 	| 'w' -> print_int Empty 1 day_of_week
 	| 'y' -> print_int pad 10 syear
 	| 'Y' -> print_int pad 1000 year
-	| 'z' -> 
+	| 'z' ->
 	    if Lazy.force tz >= 0 then print_char '+';
 	    print_int pad 10 tz;
 	    print_number fmt Zero 10 0
 	| ':' ->
-	    let idx = 
-	      try Str.search_forward (Str.regexp "z\\|:z\\|::z") f (i+1)
+	    let idx =
+	      try Re.Str.search_forward (Re.Str.regexp "z\\|:z\\|::z") f (i+1)
 	      with Not_found -> bad_format f
 	    in
-	    let next = Str.matched_string f in
+	    let next = Re.Str.matched_string f in
 	    if idx <> i+1 then bad_format f;
 	    if Lazy.force tz >= 0 then print_char '+';
 	    print_int pad 10 tz;
-	    let print_block () = 
-	      print_char ':'; 
-	      print_number fmt Zero 10 0 
+	    let print_block () =
+	      print_char ':';
+	      print_number fmt Zero 10 0
 	    in
 	    jump := String.length next;
 	    (match next with
@@ -294,10 +298,10 @@ struct
       if i = len then ()
       else match f.[i] with
       | '%' -> parse_option (i + 1) Zero
-      | c   -> 
+      | c   ->
 	  Format.pp_print_char fmt c;
 	  parse_format (i + 1)
-    in 
+    in
     parse_format 0;
     Format.pp_print_flush fmt ()
 
@@ -305,7 +309,7 @@ struct
 
   let dprint = print X.default_format
 
-  let sprint f d = 
+  let sprint f d =
     let buf = Buffer.create 15 in
     let fmt = Format.formatter_of_buffer buf in
     fprint f fmt d;
@@ -313,15 +317,15 @@ struct
 
   let to_string = sprint X.default_format
 
-  let from_fstring f s = 
+  let from_fstring f s =
     let delayed_computations = ref [] in
     let day_of_week, week = ref min_int, ref min_int in
     let year, month, day = ref min_int, ref min_int, ref min_int in
-    let hour, minute, second, pm = 
-      ref min_int, ref min_int, ref min_int, ref 0 
+    let hour, minute, second, pm =
+      ref min_int, ref min_int, ref (float min_int), ref 0
     in
     let tz = ref 0 in
-    let from_biz () = 
+    let from_biz () =
       if !week = -1 || !year = -1 then
 	bad_format (f ^ " (either week or year is not provided)");
       let d = X.from_business !year !week (Date.day_of_int !day_of_week) in
@@ -334,27 +338,33 @@ struct
     let lens = String.length s in
     let read_char c =
       if !j >= lens || s.[!j] != c then not_match f s;
-      incr j 
+      incr j
     in
     let read_number n =
       let jn = !j + n in
       if jn > lens then not_match f s;
-      let res = 
+      let res =
 	try int_of_string (String.sub s !j n)
 	with Failure _ -> not_match f s
-      in 
+      in
       j := jn;
       res
     in
-    let read_word ?(regexp=(!word_regexp)) () = 
-      let jn = 
-	try Str.search_forward regexp s !j with Not_found -> not_match f s
+    let read_word ?(regexp=(!word_regexp)) () =
+      let jn =
+	try Re.Str.search_forward regexp s !j with Not_found -> not_match f s
       in
       if jn <> !j then not_match f s;
-      let w = Str.matched_string s in
+      let w = Re.Str.matched_string s in
       j := jn + String.length w;
       w
     in
+    let read_float =
+      let regexp = Re.Str.regexp "[0-9][0-9]\\(\\.[0-9]*\\)?" in
+      fun () ->
+        try float_of_string (read_word ~regexp ())
+        with Failure _ -> not_match f s
+    in
     let parse_a () = ignore (day_of_short_name (read_word ())) in
     let parse_b () = month := month_of_short_name (read_word ()) in
     let parse_d () = day := read_number 2 in
@@ -362,13 +372,16 @@ struct
     let parse_I () = hour := read_number 2 in
     let parse_m () = month := read_number 2 in
     let parse_M () = minute := read_number 2 in
-    let parse_p () = 
+    let parse_p () =
       match read_word () with
       | "AM" -> pm := 0
       | "PM" -> pm := 12
       | s -> not_match "%p" ("\"" ^ s ^ "\"")
     in
-    let parse_S () = second := read_number 2 in
+    let parse_S () = match X.make with
+      | Int _ -> second := float (read_number 2)
+      | Float _ -> second := read_float ()
+    in
     let parse_V fmt =
       let n = read_number 2 in
       if n < 1 || n > 53 then not_match fmt (string_of_int n);
@@ -376,8 +389,8 @@ struct
     in
     let parse_y () = year := read_number 2 + 1900 in
     let parse_Y () = year := read_number 4 in
-    let parse_tz () = 
-      let sign = match read_word ~regexp:(Str.regexp "[\\+-]") () with
+    let parse_tz () =
+      let sign = match read_word ~regexp:(Re.Str.regexp "[\\+-]") () with
 	| "+" -> -1
 	| "-" -> 1
 	| _ -> assert false
@@ -385,7 +398,7 @@ struct
       let n = read_number 2 in
       tz := sign * n;
     in
-    let rec parse_option i = 
+    let rec parse_option i =
       assert (i <= lenf);
       if i = lenf then bad_format f;
       (* else *)
@@ -412,7 +425,7 @@ struct
 	   parse_Y ()
        | 'C' -> ignore (read_number 2)
        | 'd' -> parse_d ()
-       | 'D' -> 
+       | 'D' ->
 	   parse_m ();
 	   read_char '/';
 	   parse_d ();
@@ -441,7 +454,7 @@ struct
        | 'M' -> parse_M ()
        | 'n' -> read_char '\n'
        | 'p' -> parse_p ()
-       | 'P' -> 
+       | 'P' ->
 	   (match read_word () with
 	    | "am" -> pm := 0
 	    | "pm" -> pm := 12
@@ -475,7 +488,7 @@ struct
        | 'W' -> parse_V "%W"
        | 'y' -> parse_y ()
        | 'Y' -> parse_Y  ()
-       | 'z' -> 
+       | 'z' ->
 	   parse_tz ();
 	   ignore (read_number 2)
        | ':' ->
@@ -501,13 +514,16 @@ struct
       if i = lenf then begin if !j != lens then not_match f s end
       else match f.[i] with
       | '%' -> parse_option (i + 1)
-      | c -> 
+      | c ->
 	  read_char c;
 	  parse_format (i + 1)
-    in 
+    in
     parse_format 0;
     List.iter (fun f -> f ()) !delayed_computations;
-    X.make !year !month !day (!hour + !pm + !tz) !minute !second
+    let build mk = mk !year !month !day (!hour + !pm + !tz) !minute in
+    match X.make with
+    | Int f  -> build f (Utils.Float.round !second)
+    | Float f -> build f !second
 
   let from_string = from_fstring X.default_format
 
@@ -517,12 +533,13 @@ let cannot_create_event kind args =
   if List.exists ((=) min_int) args then
     raise (Invalid_argument ("Cannot create the " ^ kind))
 
-module Date = 
-  Make(struct 
+module Date =
+  Make(struct
 	 include Date
 	 let make y m d _ _ _ =
 	   cannot_create_event "date" [ y; m; d ];
 	   make y m d
+         let make = Int make
 	 let default_format = "%i"
 	 let hour _ = bad_format "hour"
 	 let minute _ = bad_format "minute"
@@ -533,12 +550,13 @@ module Date =
 
 module DatePrinter = Date
 
-module Time = 
+module Time =
   Make(struct
 	 include Time
 	 let make _ _ _ h m s =
 	   cannot_create_event "time" [ h; m; s ];
 	   make h m s
+         let make = Int make
 	 let default_format = "%T"
 	 let from_business _ _ _ = bad_format "from_business"
 	 let day_of_week _ = bad_format "day_of_week"
@@ -554,12 +572,13 @@ module Time =
 
 module TimePrinter = Time
 
-module Ftime = 
+module Ftime =
   Make(struct
 	 include Ftime
 	 let make _ _ _ h m s =
-	   cannot_create_event "time" [ h; m; s ];
-	   make h m (Second.from_int s)
+	   cannot_create_event "time" [ h; m; Utils.Float.round s ];
+	   make h m s
+         let make = Float make
 	 let second x = Second.to_int (second x)
 	 let default_format = "%T"
 	 let from_business _ _ _ = bad_format "from_business"
@@ -574,7 +593,7 @@ module Ftime =
 	 let seconds_since_1970 _ = bad_format "seconds_since_1970"
        end)
 
-module Precise_Calendar = 
+module Precise_Calendar =
   Make(struct
 	 include Calendar.Precise
 	 let make y m d h mn s =
@@ -586,9 +605,10 @@ module Precise_Calendar =
 	 let seconds_since_1970 c =
 	   let p = sub c (make 1970 1 1 0 0 0) in
 	   Time.Second.to_int (Time.Period.to_seconds (Period.to_time p))
+         let make = Int make
        end)
 
-module Calendar = 
+module Calendar =
   Make(struct
 	 include Calendar
 	 let make y m d h mn s =
@@ -600,36 +620,41 @@ module Calendar =
 	 let seconds_since_1970 c =
 	   let p = sub c (make 1970 1 1 0 0 0) in
 	   Time.Second.to_int (Time.Period.to_seconds (Period.to_time p))
+         let make = Int make
        end)
 
 module CalendarPrinter = Calendar
 
-module Precise_Fcalendar = 
+module Precise_Fcalendar =
   Make(struct
 	 include Fcalendar.Precise
 	 let make y m d h mn s =
-	   cannot_create_event "calendar" [ y; m; d; h; mn; s ];
-	   make y m d h mn (Time.Second.from_int s)
+           cannot_create_event
+             "calendar" [ y; m; d; h; mn; Utils.Float.round s ];
+	   make y m d h mn s
 	 let from_business y w d = from_date (Date.from_business y w d)
 	 let second s = Time.Second.to_int (second s)
 	 let default_format = "%i %T"
 	 let century c = Date.century (year c)
 	 let seconds_since_1970 c =
-	   let p = sub c (make 1970 1 1 0 0 0) in
+           let p = sub c (make 1970 1 1 0 0 0.) in
 	   Time.Second.to_int (Time.Period.to_seconds (Period.to_time p))
+         let make = Float make
        end)
 
-module Fcalendar = 
+module Fcalendar =
   Make(struct
 	 include Fcalendar
 	 let make y m d h mn s =
-	   cannot_create_event "calendar" [ y; m; d; h; mn; s ];
-	   make y m d h mn (Time.Second.from_int s)
+           cannot_create_event
+             "calendar" [ y; m; d; h; mn; Utils.Float.round s ];
+           make y m d h mn s
 	 let from_business y w d = from_date (Date.from_business y w d)
 	 let second s = Time.Second.to_int (second s)
 	 let default_format = "%i %T"
 	 let century c = Date.century (year c)
 	 let seconds_since_1970 c =
-	   let p = sub c (make 1970 1 1 0 0 0) in
+           let p = sub c (make 1970 1 1 0 0 0.) in
 	   Time.Second.to_int (Time.Period.to_seconds (Period.to_time p))
+         let make = Float make
        end)
