@@ -20,30 +20,28 @@
 (*  LICENSE.                                                              *)
 (**************************************************************************)
 
-(*i $Id: test_timezone.ml 259 2014-10-29 10:00:46Z signoles $ i*)
-
-Printf.printf "Tests of Time_Zone:\n";;
+(*i $Id$ i*)
 
 open CalendarLib
 open Time_Zone;;
-include Gen_test;;
-reset ();;
 
-test (current () = UTC) "current () = UTC";;
-change Local;;
-test (current () = Local) "current () = Local";;
-test (gap UTC (UTC_Plus (-5)) = -5) "gap UTC (UTC_Plus (-5)) = -5";;
-let g6 = UTC_Plus 6;;
-test 
-  (gap g6 Local = gap g6 UTC + gap UTC Local)
-  "gap g6 Local = gap g6 UTC + gap UTC Local";;
-test_exn (lazy (change (UTC_Plus 13))) "change 13";;
-test_exn (lazy (change (UTC_Plus (-15)))) "change (-15)";;
-change (UTC_Plus 4);;
-test (from_gmt () = 4) "from_gmt () = 4";;
-test (to_gmt () = -4) "to_gmt () = -4";;
+let tz = Alcotest.testable (Fmt.always "timezone") (=) ;;
 
-let ok = nb_ok ();;
-let bug = nb_bug ();;
-Printf.printf "tests ok : %d; tests ko : %d\n" ok bug;;
-flush stdout;;
+let test () =
+  change UTC;
+  Alcotest.(check tz) "current () = UTC" UTC (current ());
+  change Local;
+  Alcotest.(check tz) "current () = Local" Local (current ());
+  Alcotest.(check int) "gap UTC (UTC_Plus (-5)) = -5" ~-5 (gap UTC (UTC_Plus ~-5));
+  let g6 = UTC_Plus 6 in
+  Alcotest.(check int)
+    "gap g6 Local = gap g6 UTC + gap UTC Local"
+    (gap g6 Local) (gap g6 UTC + gap UTC Local);
+  Gen_test.test_exn (lazy (change (UTC_Plus 13))) "change 13";
+  Gen_test.test_exn (lazy (change (UTC_Plus (-15)))) "change (-15)";
+  change (UTC_Plus 4);
+  Alcotest.(check int) "from_gmt () = 4" 4 (from_gmt ()) ;
+  Alcotest.(check int) "to_gmt () = -4" ~-4 (to_gmt ());
+  ()
+
+let suite = ["timezone", `Quick, test]
